@@ -22,4 +22,10 @@ baseline = function (...)
     argname <- ifelse(names(allf)=="", words[1:nterms], names(allf));
   }
   allf
+}
+
+frailtyprior = function (prior="car", ...) {
+  x = cbind(...);
+  colnames(x)=rep(prior,ncol(x));
+  x
 }
\ No newline at end of file
